#include <string.h>

#include "http_loop.h"
#include "log.h"

http_parse_error http_loop_parse_request(event_loop_conn *conn) {
  http_loop_ctx *ctx = conn->ctx;

  // First we try to parse the incoming HTTP request
  size_t num_headers = HTTP_MAX_ALLOWED_HEADERS;
  http_request *req = &ctx->req;

  const char *method;
  size_t method_len;

  int res = phr_parse_request(
      (const char *)&conn->rbuf[conn->rbuf_read],
      conn->rbuf_size - conn->rbuf_read, &method, &method_len, &req->path,
      &req->path_len, &req->minor_version, req->headers, &num_headers, 0);

  if (res == -1) {
    return http_parse_error_invalid;
  } else if (res == -2) {
    return http_parse_error_incomplete;
  }

  req->len = res;

  // Try to parse the method type
  bool match = false;
  size_t i = 0;

  for (i = 0; i < request_method_names_len; i++) {
    if (strncmp(method, request_method_names[i], method_len) == 0) {
      req->method = i;
      match = true;
    }
  }

  if (!match) {
    return http_parse_error_unknown_method;
  }

  // Split path into path & query
  i = 0;
  bool no_query = true;

  while (no_query && i < req->path_len) {
    if (req->path[i] == '?') {
      // Ensure we don't store an invalid pointer if the request simply ends
      // with '?'
      if (i + 1 < req->path_len) {
        req->query = &req->path[i + 1];
        req->query_len = req->path_len - (i + 1);
      }

      req->path_len = i;

      no_query = false;
    }

    i++;
  }

  // Ensure we clear the old request's query
  if (no_query) {
    req->query = NULL;
    req->query_len = 0;
  }

  return http_parse_error_ok;
}

void http_loop_route_request(event_loop_conn *conn) {
  http_loop_ctx *ctx = conn->ctx;
  http_loop_gctx *gctx = ctx->g;

  info("%s %.*s", request_method_names[ctx->req.method], ctx->req.path_len,
       ctx->req.path);

  http_route *route;

  for (size_t i = 0; i < gctx->route_count; i++) {
    route = &gctx->routes[i];

    switch (route->type) {
    case http_route_literal:
      if (strncmp(route->path, ctx->req.path, ctx->req.path_len) == 0) {
        ctx->route = route;
        return;
      }
    // TODO
    case http_route_regex:;
    }
  }

  // Fallthrough is to write a 404
  ctx->res.type = http_not_found;
  conn->state = event_loop_conn_state_res;
}

void http_loop_process_request(event_loop_conn *conn) {
  http_loop_ctx *ctx = conn->ctx;

  // We keep processing step functions as long as they don't need to wait for
  // I/O
  while ((conn->state == event_loop_conn_state_req) &&
         (ctx->route->steps[ctx->current_step] != NULL) &&
         ctx->route->steps[ctx->current_step](conn)) {
    ctx->current_step++;
  }

  if (conn->state != event_loop_conn_state_req) {
    return;
  }

  // If we've reached the end of the list of step functions, we report the
  // request as finished by clearing its route
  if (ctx->route->steps[ctx->current_step] == NULL) {
    http_loop_ctx_reset(ctx);
  }
}
